#!/bin/bash

#set -x

DIRP=`readlink -f $0`
PWD=`dirname $DIRP`
PKG_FOLDER=/opt/tct/packages
RES_FOLDER=/opt/tct/resource
SCR_FOLDER=/opt/tct/scripts
INSTALLER="dpkg"
PKG_MGR="apt-get install"
PLATFORM=`uname -m`
SYS_BIN=""
SDB_SRC=""


if [ -d /usr/bin ]; then
    SYS_BIN="/usr/bin"
elif [ -d /bin ]; then
    SYS_BIN="/bin"
else
    echo "WARNING: cannot access /usr/bin or /bin to configure sdb"
    echo "         Please manually install sdb, test packages, test manager and configure packages"
    exit 255
fi


function show_usage(){
    echo "Usage: $0 command [parameters]"
    echo "commands:"
    echo "      sudo $0 --install      - Install tct tools on host, '--install' can be omitted." 
    echo "      sudo $0 --purge/-p     - Clean up tct tools installed on host."
    exit 0
}

function config_sdb(){
    # set up sdb
    echo "Installing sdb tool, please wait..."
    cp $SDB_SRC $SYS_BIN/sdb -f > /dev/null 2>&1
    chmod 777 $SYS_BIN/sdb
    ${PKG_MGR} libudev0 > /dev/null 2>&1

    which sdb > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "WARNING: configure sdb $SYS_BIN failed"
        echo "         Please manually install sdb, test packages, test manager and configure packages"
        exit 255
    else
        echo "sdb is installed successfully"
    fi      
}

function install_sdb(){
    if [ $PLATFORM = "i686" -o $PLATFORM = "i386" ]; then
        SDB_SRC=$PWD/sdb-32
    elif [ $PLATFORM = "x86_64" ]; then
        SDB_SRC=$PWD/sdb-64
    else
        echo "WARNING: cannot deploy correct sdb to host machine"
        echo "         Please manually install sdb, test packages, test manager and configure packages"
        exit 255
    fi

    # check sdb version
    SDB_VERSION=`sdb version 2>/dev/null | cut -d ' ' -f5`
    INS_SDB_VERSION=`$SDB_SRC version | cut -d ' ' -f5`
    if [ -z "$SDB_VERSION" ]; then
        config_sdb
    else
        read -p "CONFIRM: sdb version $SDB_VERSION exists in Host, replace it with version $INS_SDB_VERSION or not[Y/N]:" ANSWER
        if [ $ANSWER == "Y" -o $ANSWER == "y" ]; then
            config_sdb
        else
            echo "sdb installation skipped, keep current sdb unchange."
        fi
    fi
}

function keep_or_remove(){
    if [ -d $1 ]; then
        read -p "CONFIRM: install script find existed folder $1, choose 'Y/y' to clean the files[Y/N]:" ANSWER
        if [ $ANSWER == "Y" -o $ANSWER == "y" ]; then
            rm $1/* -rf
        else
            echo "WARNING: Won't clean up existed files in $1. Overwrite old ones without notification"
        fi
    else
        mkdir -p $1
    fi    
}

function install_resources(){
    # set up /opt/tct/packages folder for test packages
    echo "try to set up $PKG_FOLDER folder for test packages"
    keep_or_remove $PKG_FOLDER
    cp $PWD/../package/tct-* $PKG_FOLDER -f > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        rm $PKG_FOLDER/tct-behavior* -f > /dev/null 2>&1
	rm $PKG_FOLDER/tct-testconfig* -f > /dev/null 2>&1
        echo "copy packages successfully"
    else
        echo "WARNING: copy test packages to $PKG_FOLDER failed. Please copy test packages in package folder to $PKG_FOLDER manually"
        exit 255
    fi

    cp $PWD/../package/package_list.xml $PKG_FOLDER -f > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "copy package_list.xml successfully"
    else
        echo "WARNING: copy package_list.xml to $PKG_FOLDER failed. Please copy package_list.xml to $PKG_FOLDER manually"
        exit 255
    fi

    # set up /opt/tct/resource folder
    echo "try to set up $RES_FOLDER folder for test resources"
    keep_or_remove $RES_FOLDER
    cp $PWD/../resource/* $RES_FOLDER  -R -f > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "copy resource folder successfully"
    else
        echo "WARNING: copy test resources to $RES_FOLDER failed. Please copy test resources in resource folder to $RES_FOLDER manually"
        exit 255
    fi
    # copy test config to /opt/tct/resource
    RPMPKG=`ls ../package/ | grep tct-testconfig`
    cp $PWD/../package/$RPMPKG $RES_FOLDER  -R -f > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "copy testconfig successfully"
    else
        echo "WARNING: copy testconfig to $RES_FOLDER failed. Please copy test resources in resource folder to $RES_FOLDER manually"
        exit 255
    fi

    # set up /opt/tct/scripts folder
    echo "try to set up $SCR_FOLDER folder for test scripts"
    keep_or_remove $SCR_FOLDER
    cp $PWD/tct-config-device.sh $SCR_FOLDER -f > /dev/null 2>&1
    cp $PWD/healthcheck.ini $SCR_FOLDER -f > /dev/null 2>&1
    chmod ugo+rwx $SCR_FOLDER/tct-config-device.sh > /dev/null 2>&1
    chmod ugo+rwx $SCR_FOLDER/healthcheck.ini > /dev/null 2>&1
    echo "copy scripts successfully"
}

function uninstall_pkg(){
    ${INSTALLER} -l | grep $1 > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "you have installed $1 on your machine. Now try to uninstall it"
        ${INSTALLER} -r $1 > /dev/null 2>&1
        if [ $? -ne 0 ];then
            echo "WARNING:$1 uninstall failed. please uninstall it manually using software center or following command:"
            echo "    sudo dpkg -r $1"
            exit 255
        else
            echo "$1 uninstalled successfully"
        fi
    else
        echo "no $1 is installed on your machine"
    fi
}

function install_dependencies(){
    # check JDK
    echo "Configuring dependencies for tct, please wait..."
    read -p "Java runtime: openJDK 7 or JDK 6(above) is required. Please confirm JDK configured properly[Y/N]:" ANSWER
    if [ $ANSWER == "Y" -o $ANSWER == "y" ]; then
        echo "continue"
    else
        echo "WARNING: JDK is not configured. Please configure it first."
        echo "You can download it from network java.sun.com or http://www.oracle.com, and configure it in host accroding guide in network"
        echo "You can check the version via command:"
        echo "java -version or"
        echo "which java"
        exit 255
    fi

    # check python & tools
    ${PKG_MGR} rpm2cpio tree python-support
    if [ $? -eq 0 ]; then
        echo "Dependencies are configured successfully"
    else
        echo "WARNING: tct-shell and testkit-lite dependency configuration failed. Please check whether network is available and apt-get can work "
        echo "         and make sure the dependencies below installed properly using <sudo apt-get install ***> "
        echo "           rpm2cpio"
        echo "           tree"
        echo "           python-support"
        echo "           timeout"
        echo "And then re-run the install script"
        exit 255 
    fi    
}

function clean_host(){
    rm -rf $PKG_FOLDER/ > /dev/null 2>&1
    rm -rf $SCR_FOLDER/ > /dev/null 2>&1
    rm -rf $RES_FOLDER/ > /dev/null 2>&1
    rm -rf /opt/testkit > /dev/null 2>&1
    rm -rf /opt/tct > /dev/null 2>&1
    rm -rf /usr/lib/python2.7/dist-packages/commodule > /dev/null 2>&1
    rm -rf /usr/lib/python2.7/dist-packages/testkitlite > /dev/null 2>&1
    rm -rf /usr/lib/python2.7/dist-packages/tctshell > /dev/null 2>&1
    rm -rf /usr/share/pyshared/commodule > /dev/null 2>&1
    rm -rf /usr/share/pyshared/testkitlite > /dev/null 2>&1
    rm -rf /usr/share/pyshared/tctshell > /dev/null 2>&1
}

function clean_manager(){
    rm -rf /opt/tct/manager
    rm -rf /usr/bin/tct-mgr
    rm -rf /usr/bin/full_pkg_generator
}

function install_manager(){
    echo "Install tct-manager, please wait..."
    mkdir -p /opt/tct
    mkdir -p /opt/tct/manager
    mkdir -p /opt/tct/manager/lib
    mkdir -p /opt/tct/manager/plan
    mkdir -p /opt/tct/manager/style
    mkdir -p /opt/tct/manager/doc
    mkdir -p /opt/tct/manager/log
    mkdir -p /opt/tct/manager/config
    mkdir -p /opt/tct/manager/result
    chmod ugo+rwx /opt/tct/manager
    chmod ugo+rwx /opt/tct/manager/lib
    chmod ugo+rwx /opt/tct/manager/plan
    chmod ugo+rwx /opt/tct/manager/style
    chmod ugo+rwx /opt/tct/manager/log
    chmod ugo+rwx /opt/tct/manager/config
    chmod ugo+rwx /opt/tct/manager/result

    cp $PWD/tct-mgr/dest/*.jar /opt/tct/manager/lib/
    cp $PWD/tct-mgr/dest/lib/*.jar /opt/tct/manager/lib/
    cp $PWD/tct-mgr/dest/tct-mgr /usr/bin/    
    cp $PWD/tct-mgr/dest/full_pkg_generator /usr/bin/
    cp $PWD/tct-mgr/style/* /opt/tct/manager/style/
    cp $PWD/tct-mgr/doc/* /opt/tct/manager/doc/
    cp $PWD/tct-mgr/dest/log4j.properties /opt/tct/manager/config/
    cp $PWD/Full_test_mgr.xml /opt/tct/manager/plan/Full_test.xml -f > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Copy Full_test_mgr.xml to /opt/tct/manager/plan/ failed. Please copy this file manually"
        exit 255
    fi
    chmod ugo+rx /usr/bin/tct-mgr
    chmod ugo+rx /usr/bin/full_pkg_generator
    chmod ugo+rwx /opt/tct/packages > /dev/null 2>&1
    chmod ugo+rwx /opt/tct/manager/plan/Full_test.xml > /dev/null 2>&1
    chmod ugo+rwx /opt/tct/packages/package_list.xml > /dev/null 2>&1
}

function clean_testkitlite(){
    ${INSTALLER} -l | grep testkit-lite > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "you have installed testkit-lite on your machine, now uninstall it"
        ${INSTALLER} -r testkit-lite > /dev/null 2>&1
        if [ $? -ne 0 ];then
            echo "WARNING:testkit-lite uninstall failed. please uninstall it manually using software center or following command:"
            echo "    sudo dpkg -r testkit-lite"
            exit 255
        fi
    fi

    echo "clear old folders created by testkit-lite, please wait..."
    rm -rf /usr/lib/python2.7/dist-packages/commodule > /dev/null 2>&1
    rm -rf /usr/lib/python2.7/dist-packages/testkitlite > /dev/null 2>&1
    rm -rf /usr/share/pyshared/commodule > /dev/null 2>&1
    rm -rf /usr/share/pyshared/testkitlite > /dev/null 2>&1
    rm -rf /opt/testkit/lite > /dev/null 2>&1    
}

function install_testkitlite(){
    echo "install testkit-lite, please wait..."
    ${PKG_MGR} python-pip
    if [ $? -eq 0 ]; then
        QUE_VER=`pip freeze | grep requests==1.[123456789].[0123456789] | cut -d = -f 3`
        if [ -z "${QUE_VER}" ]; then
            echo "Configuring testkit-lite dependency: python-requests, please wait..."
            pip install --upgrade requests > /dev/null 2>&1
            if [ $? -ne 0 ]; then
            echo "WARNING: testkit-lite dependency requests configuration failed"
            echo "         please install it manually by command: "
                echo "             sudo pip install requests "
                exit 255
            fi
        fi
    else
        echo "WARNING: testkit-lite dependency is not completed because of pip installed failed."
        echo "please make sure pip installed properly by command: "
        echo "sudo apt-get install python-pip"
        echo "And then install requests by command:"
        echo "sudo pip install requests "
        exit 255 
    fi

    DKG_SRC=`ls $PWD/ | grep testkit-lite`
    ${INSTALLER} -i $PWD/$DKG_SRC > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "testkit-lite is installed successfully"
    else
        echo "WARNING: install testkit-lite failed, please go to $PWD to install testkit-lite manually"
        exit 255
    fi    
}

function clean_shell(){
    ${INSTALLER} -l | grep tct-shell > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        echo "you have installed tct-shell on your machine. Will uninstall it first and then install again"
        ${INSTALLER} -r tct-shell > /dev/null 2>&1
        if [ $? -ne 0 ];then
            echo "WARNING:tct-shell uninstall failed. please uninstall it manually using software center or following command:"
            echo "    sudo dpkg -r tct-shell"
            exit 255
        fi
    fi
    echo "clear old folders created by tct-shell, please wait..."
    rm -rf /usr/lib/python2.7/dist-packages/tctshell > /dev/null 2>&1    
    rm -rf /usr/share/pyshared/tctshell > /dev/null 2>&1
}

function install_shell(){
    echo "install tct-shell, please wait..."
    TCT_SHELL=`ls $PWD/ | grep tct-shell`
    ${INSTALLER} -i $PWD/$TCT_SHELL > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        tct-shell > /dev/null 2>&1
        cp $PWD/Full_auto_shell.xml /opt/tct/shell/plan/Full_auto.xml -f > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            echo "Copy Full_auto_shell.xml to /opt/tct/shell/plan/ failed. Please copy this file manually"
            exit 255
        fi
    else
        echo "Install tct-shell failed first time. Install dependencies and re-install tct-shell again"
        apt-get -f install
        if [ $? -ne 0 ]; then
            echo "WARNING: install tct-shell failed again, please configure dependencies using <sudo apt-get install python-support libtool debhelper> and go to $PWD to install tct-shell manually"
            exit 255
        else
             tct-shell > /dev/null 2>&1
             cp $PWD/Full_auto_shell.xml /opt/tct/shell/plan/Full_auto.xml -f > /dev/null 2>&1
             if [ $? -ne 0 ]; then
                echo "Copy Full_auto_shell.xml to /opt/tct/shell/plan/ failed. Please copy this file manually"
                exit 255
             fi
        fi
    fi    
}

function install_host(){
    echo "------------------------------------------------------------------------"
    echo "[install stage 1: configure JDK, sdb and test packages on host machine...]"
    install_dependencies && install_sdb && install_resources 

    echo "------------------------------------------------------------------------"
    echo "[install stage 2: install tct-manager on host machine...]"
    clean_manager && install_manager

    echo "------------------------------------------------------------------------"
    echo "[install stage 3: install tct-shell on host machine...]"
    clean_shell && install_shell

    echo "------------------------------------------------------------------------"
    echo "[install stage 4: install testkit-lite on host machine...]"
    clean_testkitlite && install_testkitlite

    echo "All of host installation process is DONE"
    exit 0
}

if [ x$1 != x ]; then
    if [ $1 = "-p" -o $1 = "--purge" ]; then
        echo "----------------------------------------------"
        echo "Purge installed tct tools on host, please wait..."
        uninstall_pkg "tct-shell"
        uninstall_pkg "testkit-lite"
        clean_manager
        clean_host
        echo "Purge tools on host successfully"
        echo "----------------------------------------------"
        exit 0
    elif [ $1 = "--install" ]; then
        install_host
    else
        show_usage
    fi
else
    install_host
fi
